@@ -1,84 +1,40 @@
-# ----------------------------------------------------------------------------
-# pyglet
-# Copyright (c) 2006-2008 Alex Holkner
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-#  * Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in
-#    the documentation and/or other materials provided with the
-#    distribution.
-#  * Neither the name of pyglet nor the names of its
-#    contributors may be used to endorse or promote products
-#    derived from this software without specific prior written
-#    permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
-# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
-# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-# ----------------------------------------------------------------------------
+"""pyglet is a cross-platform games and multimedia package.
 
-'''pyglet is a cross-platform games and multimedia package.
-
-Detailed documentation is available at http://www.pyglet.org
-'''
-
-__docformat__ = 'restructuredtext'
-__version__ = '$Id$'
+More information is available at http://www.pyglet.org
+"""
 
 import os
 import sys
 
-_is_epydoc = hasattr(sys, 'is_epydoc') and sys.is_epydoc
+from typing import TYPE_CHECKING
 
-#: The release version of this pyglet installation.
-#:
-#: Valid only if pyglet was installed from a source or binary distribution
-#: (i.e. not in a checked-out copy from SVN).
-#:
-#: Use setuptools if you need to check for a specific release version, e.g.::
-#:
-#:    >>> import pyglet
-#:    >>> from pkg_resources import parse_version
-#:    >>> parse_version(pyglet.version) >= parse_version('1.1')
-#:    True
-#:
-version = '1.2.3'
+#: The release version
+version = '2.0.12'
+__version__ = version
 
-# Pyglet platform treats *BSD systems as Linux
+MIN_PYTHON_VERSION = 3, 8
+MIN_PYTHON_VERSION_STR = '.'.join([str(v) for v in MIN_PYTHON_VERSION])
+
+if sys.version_info < MIN_PYTHON_VERSION:
+    raise Exception(f"pyglet {version} requires Python {MIN_PYTHON_VERSION_STR} or newer.")
+
+if 'sphinx' in sys.modules:
+    setattr(sys, 'is_pyglet_doc_run', True)
+_is_pyglet_doc_run = hasattr(sys, "is_pyglet_doc_run") and sys.is_pyglet_doc_run
+
+# pyglet platform treats *BSD systems as Linux
 compat_platform = sys.platform
 if "bsd" in compat_platform:
     compat_platform = "linux-compat"
 
-def _require_ctypes_version(version):
-    # Check ctypes version
-    import ctypes
-    req = [int(i) for i in version.split('.')]
-    have = [int(i) for i in ctypes.__version__.split('.')]
-    if not tuple(have) >= tuple(req):
-        raise ImportError('pyglet requires ctypes %s or later.' % version)
-_require_ctypes_version('1.0.0')
-
 _enable_optimisations = not __debug__
 if getattr(sys, 'frozen', None):
     _enable_optimisations = True
 
-#: Global dict of pyglet options.  To change an option from its default, you
-#: must import ``pyglet`` before any sub-packages.  For example::
+#: Global dict of pyglet options.
+#:
+#: To change an option from its default, you must import
+#: ``pyglet`` before any sub-packages.  For example::
 #:
 #:      import pyglet
 #:      pyglet.options['debug_gl'] = False
@@ -94,13 +50,21 @@ if getattr(sys, 'frozen', None):
 #: The non-development options are:
 #:
 #: audio
-#:     A sequence of the names of audio modules to attempt to load, in
-#:     order of preference.  Valid driver names are:
+#:     A :py:class:`~typing.Sequence` of valid audio modules names. They will
+#:     be tried from first to last until either a driver loads or no entries
+#:     remain. See :ref:`guide-audio-driver-order` for more information.
+#:
+#:     Valid driver names are:
+#:
+#:     * ``'xaudio2'``, the Windows Xaudio2 audio module (Windows only)
+#:     * ``'directsound'``, the Windows DirectSound audio module (Windows only)
+#:     * ``'pulse'``, the :ref:`guide-audio-driver-pulseaudio` module
+#:        (Linux only, otherwise nearly ubiquitous. Limited features; use
+#:        ``'openal'`` for more.)
+#:     * ``'openal'``, the :ref:`guide-audio-driver-openal` audio module
+#:       (A library may need to be installed on Windows and Linux)
+#:     * ``'silent'``, no audio
 #:
-#:     * directsound, the Windows DirectSound audio module (Windows only)
-#:     * pulse, the PulseAudio module (Linux only)
-#:     * openal, the OpenAL audio module
-#:     * silent, no audio
 #: debug_lib
 #:     If True, prints the path of each dynamic library loaded.
 #: debug_gl
@@ -123,7 +87,7 @@ if getattr(sys, 'frozen', None):
 #:     must be loaded after the window using them was created).  Recommended
 #:     for advanced developers only.
 #:
-#:     **Since:** pyglet 1.1
+#:     .. versionadded:: 1.1
 #: vsync
 #:     If set, the `pyglet.window.Window.vsync` property is ignored, and
 #:     this option overrides it (to either force vsync on or off).  If unset,
@@ -137,30 +101,22 @@ if getattr(sys, 'frozen', None):
 #:     X11 servers supporting the Xsync extension with a window manager
 #:     that implements the _NET_WM_SYNC_REQUEST protocol.
 #:
-#:     **Since:** pyglet 1.1
-#: darwin_cocoa
-#:     If True, the Cocoa-based pyglet implementation is used as opposed to
-#:     the 32-bit Carbon implementation.  When python is running in 64-bit mode
-#:     on Mac OS X 10.6 or later, this option is set to True by default.
-#:     Otherwise the Carbon implementation is preferred.
-#:
-#:     **Since:** pyglet 1.2
-#:
+#:     .. versionadded:: 1.1
 #: search_local_libs
 #:     If False, pyglet won't try to search for libraries in the script
 #:     directory and its `lib` subdirectory. This is useful to load a local
 #:     library instead of the system installed version. This option is set
 #:     to True by default.
 #:
-#:     **Since:** pyglet 1.2
+#:     .. versionadded:: 1.2
 #:
 options = {
-    'audio': ('directsound', 'pulse', 'openal', 'silent'),
-    'font': ('gdiplus', 'win32'), # ignored outside win32; win32 is deprecated
+    'audio': ('xaudio2', 'directsound', 'openal', 'pulse', 'silent'),
     'debug_font': False,
     'debug_gl': not _enable_optimisations,
     'debug_gl_trace': False,
     'debug_gl_trace_args': False,
+    'debug_gl_shaders': False,
     'debug_graphics_batch': False,
     'debug_lib': False,
     'debug_media': False,
@@ -170,23 +126,30 @@ options = {
     'debug_trace_depth': 1,
     'debug_trace_flush': True,
     'debug_win32': False,
+    'debug_input': False,
     'debug_x11': False,
-    'graphics_vbo': True,
     'shadow_window': True,
     'vsync': None,
     'xsync': True,
     'xlib_fullscreen_override_redirect': False,
-    'darwin_cocoa': False,
     'search_local_libs': True,
+    'win32_gdi_font': False,
+    'headless': False,
+    'headless_device': 0,
+    'win32_disable_shaping': False,
+    'dw_legacy_naming': False,
+    'win32_disable_xinput': False,
+    'com_mta': False,
+    'osx_alt_loop': False
 }
 
 _option_types = {
     'audio': tuple,
-    'font': tuple,
     'debug_font': bool,
     'debug_gl': bool,
     'debug_gl_trace': bool,
     'debug_gl_trace_args': bool,
+    'debug_gl_shaders': bool,
     'debug_graphics_batch': bool,
     'debug_lib': bool,
     'debug_media': bool,
@@ -196,71 +159,69 @@ _option_types = {
     'debug_trace_depth': int,
     'debug_trace_flush': bool,
     'debug_win32': bool,
+    'debug_input': bool,
     'debug_x11': bool,
-    'graphics_vbo': bool,
     'shadow_window': bool,
     'vsync': bool,
     'xsync': bool,
     'xlib_fullscreen_override_redirect': bool,
-    'darwin_cocoa': bool,
+    'search_local_libs': bool,
+    'win32_gdi_font': bool,
+    'headless': bool,
+    'headless_device': int,
+    'win32_disable_shaping': bool,
+    'dw_legacy_naming': bool,
+    'win32_disable_xinput': bool,
+    'com_mta': bool,
+    'osx_alt_loop': bool,
 }
 
-def _choose_darwin_platform():
-    """Choose between Darwin's Carbon and Cocoa implementations."""
-    if compat_platform != 'darwin':
-        return
-    import struct
-    numbits = 8*struct.calcsize("P")
-    if numbits == 64:
-        import platform
-        osx_version = platform.mac_ver()[0].split(".")
-        if int(osx_version[0]) == 10 and int(osx_version[1]) < 6:
-            raise Exception('pyglet is not compatible with 64-bit Python for versions of Mac OS X prior to 10.6.')
-        options['darwin_cocoa'] = True
-    else:
-        options['darwin_cocoa'] = False
-_choose_darwin_platform()  # can be overridden by an environment variable below
 
-def _read_environment():
-    '''Read defaults for options from environment'''
-    for key in options:
-        env = 'PYGLET_%s' % key.upper()
-        try:
-            value = os.environ[env]
-            if _option_types[key] is tuple:
-                options[key] = value.split(',')
-            elif _option_types[key] is bool:
-                options[key] = value in ('true', 'TRUE', 'True', '1')
-            elif _option_types[key] is int:
-                options[key] = int(value)
-        except KeyError:
-            pass
-_read_environment()
+for _key in options:
+    """Check Environment Variables for pyglet options"""
+    assert _key in _option_types, f"Option '{_key}' must have a type set in _option_types."
+
+    if _value := os.environ.get(f'PYGLET_{_key.upper()}'):
+        if _option_types[_key] is tuple:
+            options[_key] = _value.split(',')
+        elif _option_types[_key] is bool:
+            options[_key] = _value in ('true', 'TRUE', 'True', '1')
+        elif _option_types[_key] is int:
+            options[_key] = int(_value)
+
 
 if compat_platform == 'cygwin':
     # This hack pretends that the posix-like ctypes provides windows
     # functionality.  COM does not work with this hack, so there is no
     # DirectSound support.
     import ctypes
+
     ctypes.windll = ctypes.cdll
     ctypes.oledll = ctypes.cdll
     ctypes.WINFUNCTYPE = ctypes.CFUNCTYPE
     ctypes.HRESULT = ctypes.c_long
 
+
 # Call tracing
 # ------------
 
 _trace_filename_abbreviations = {}
+_trace_thread_count = 0
+_trace_args = options['debug_trace_args']
+_trace_depth = options['debug_trace_depth']
+_trace_flush = options['debug_trace_flush']
+
 
 def _trace_repr(value, size=40):
     value = repr(value)
     if len(value) > size:
-        value = value[:size//2-2] + '...' + value[-size//2-1:]
+        value = value[:size // 2 - 2] + '...' + value[-size // 2 - 1:]
     return value
 
+
 def _trace_frame(thread, frame, indent):
-    from pyglet import lib
-    if frame.f_code is lib._TraceFunction.__call__.func_code:
+
+    if frame.f_code is lib._TraceFunction.__call__.__code__:
         is_ctypes = True
         func = frame.f_locals['self']._func
         name = func.__name__
@@ -276,39 +237,40 @@ def _trace_frame(thread, frame, indent):
             filename = _trace_filename_abbreviations[path]
         except KeyError:
             # Trim path down
-            dir = ''
+            directory = ''
             path, filename = os.path.split(path)
-            while len(dir + filename) < 30:
-                filename = os.path.join(dir, filename)
-                path, dir = os.path.split(path)
-                if not dir:
-                    filename = os.path.join('', filename)
+
+            while len(directory + filename) < 30:
+                filename = os.path.join(directory, filename)
+                path, directory = os.path.split(path)
+                if not directory:
                     break
             else:
                 filename = os.path.join('...', filename)
             _trace_filename_abbreviations[path] = filename
 
-        location = '(%s:%d)' % (filename, line)
+        location = f'({filename}:{line})'
 
     if indent:
-        name = 'Called from %s' % name
-    print '[%d] %s%s %s' % (thread, indent, name, location)
+        name = f'Called from {name}'
+    print(f'[{thread}] {indent}{name} {location}')
 
     if _trace_args:
         if is_ctypes:
             args = [_trace_repr(arg) for arg in frame.f_locals['args']]
-            print '  %sargs=(%s)' % (indent, ', '.join(args))
+            print(f'  {indent}args=({", ".join(args)})')
         else:
             for argname in code.co_varnames[:code.co_argcount]:
                 try:
                     argvalue = _trace_repr(frame.f_locals[argname])
-                    print '  %s%s=%s' % (indent, argname, argvalue)
+                    print(f'  {indent}{argname}={argvalue}')
                 except:
                     pass
 
     if _trace_flush:
         sys.stdout.flush()
 
+
 def _thread_trace_func(thread):
     def _trace_func(frame, event, arg):
         if event == 'call':
@@ -322,25 +284,21 @@ def _thread_trace_func(thread):
 
         elif event == 'exception':
             (exception, value, traceback) = arg
-            print 'First chance exception raised:', repr(exception)
+            print('First chance exception raised:', repr(exception))
+
     return _trace_func
 
+
 def _install_trace():
     global _trace_thread_count
     sys.setprofile(_thread_trace_func(_trace_thread_count))
     _trace_thread_count += 1
 
-_trace_thread_count = 0
-_trace_args = options['debug_trace_args']
-_trace_depth = options['debug_trace_depth']
-_trace_flush = options['debug_trace_flush']
-if options['debug_trace']:
-    _install_trace()
 
 # Lazy loading
 # ------------
 
-class _ModuleProxy(object):
+class _ModuleProxy:
     _module = None
 
     def __init__(self, name):
@@ -353,7 +311,7 @@ class _ModuleProxy(object):
             if self._module is not None:
                 raise
 
-            import_name = 'pyglet.%s' % self._module_name
+            import_name = f'pyglet.{self._module_name}'
             __import__(import_name)
             module = sys.modules[import_name]
             object.__setattr__(self, '_module', module)
@@ -367,51 +325,60 @@ class _ModuleProxy(object):
             if self._module is not None:
                 raise
 
-            import_name = 'pyglet.%s' % self._module_name
+            import_name = f'pyglet.{self._module_name}'
             __import__(import_name)
             module = sys.modules[import_name]
             object.__setattr__(self, '_module', module)
             globals()[self._module_name] = module
             setattr(module, name, value)
 
-if True:
+
+# Lazily load all modules, except if performing
+# type checking or code inspection.
+if TYPE_CHECKING:
+    from . import app
+    from . import canvas
+    from . import clock
+    from . import customtypes
+    from . import event
+    from . import font
+    from . import gl
+    from . import graphics
+    from . import gui
+    from . import input
+    from . import image
+    from . import lib
+    from . import math
+    from . import media
+    from . import model
+    from . import resource
+    from . import sprite
+    from . import shapes
+    from . import text
+    from . import window
+else:
     app = _ModuleProxy('app')
     canvas = _ModuleProxy('canvas')
     clock = _ModuleProxy('clock')
-    com = _ModuleProxy('com')
+    customtypes = _ModuleProxy('customtypes')
     event = _ModuleProxy('event')
     font = _ModuleProxy('font')
     gl = _ModuleProxy('gl')
     graphics = _ModuleProxy('graphics')
+    gui = _ModuleProxy('gui')
     image = _ModuleProxy('image')
     input = _ModuleProxy('input')
     lib = _ModuleProxy('lib')
+    math = _ModuleProxy('math')
     media = _ModuleProxy('media')
+    model = _ModuleProxy('model')
     resource = _ModuleProxy('resource')
     sprite = _ModuleProxy('sprite')
+    shapes = _ModuleProxy('shapes')
     text = _ModuleProxy('text')
     window = _ModuleProxy('window')
 
-# Fool py2exe, py2app into including all top-level modules (doesn't understand
-# lazy loading)
-if False:
-    import app
-    import canvas
-    import clock
-    import com
-    import event
-    import font
-    import gl
-    import graphics
-    import input
-    import image
-    import lib
-    import media
-    import resource
-    import sprite
-    import text
-    import window
 
-# Hack around some epydoc bug that causes it to think pyglet.window is None.
-if False:
-    import window
+# Call after creating proxies:
+if options['debug_trace']:
+    _install_trace()
